# encoding = utf - 8

import numpy as np


class LossPrinter(object):
    def __init__(self, run_name, end_step):
        self.run_name = run_name
        self.end_step = end_step
        self.rloss_several_steps = []
        self.gam_loss_several_steps = []
        self.regression_loss_several_steps = []

        self.batch_correct_count = 0.0
        self.batch_count = 0.0

    def update(self, net_outputs):
        if 'classification' not in net_outputs:
            self.rloss_several_steps.append(net_outputs['losses']['refine_loss0'].item()
                                            + net_outputs['losses']['refine_loss1'].item()
                                            + net_outputs['losses']['refine_loss2'].item())

            if 'gam_loss_im_cls' in net_outputs['losses']:
                self.gam_loss_several_steps.append(net_outputs['losses']['gam_loss_im_cls'].item())

            if 'regression_loss' in net_outputs['losses']:
                self.regression_loss_several_steps.append(net_outputs['losses']['regression_loss'].item())
        else:
            self.batch_correct_count += net_outputs['classification']['correct_count']
            self.batch_count += net_outputs['classification']['total_count']

    def print_loss_status(self, step, loss_several_steps):
        gam_loss_show, regression_loss_show = 0.0, 0.0
        if len(self.gam_loss_several_steps) > 0:
            gam_loss_show = round(np.median(self.gam_loss_several_steps).item(), 6)
        if len(self.regression_loss_several_steps) > 0:
            regression_loss_show = round(np.median(self.regression_loss_several_steps).item(), 6)

        print(f"[{self.run_name}] [Step {step + 1}/{self.end_step}] " +
              f"loss: {round(np.median(loss_several_steps).item(), 6)}  " +
              f"r_loss: {round(np.median(self.rloss_several_steps).item(), 6)} " +
              f"gam_loss: {gam_loss_show} " +
              f"regression_loss: {regression_loss_show}")

        self.rloss_several_steps = []
        self.gam_loss_several_steps = []
        self.regression_loss_several_steps = []

        if self.batch_count > 0:
            print(f"prediction precision of batch: {round(self.batch_correct_count / self.batch_count, 3)}")
            self.batch_count, self.batch_correct_count = 0.0, 0.0
